@@ -6,12 +6,12 @@ config = {
     "embedding_size": 768,
     "initializer_range": 0.02,
     "intermediate_size": 3072,
-    "max_position_embeddings": 512,
+    "max_position_embeddings": -1,
     "num_attention_heads": 12,
     "attention_head_size": 64,
     "num_hidden_layers": 12,
     "vocab_size": 32128,
-    "type_vocab_size": 0,
+    "type_vocab_size": -1,
     "layer_norm_epsilon": 1e-06,
     "bidirectional": True,
     "positional_buckets": 32,
